#
# PST module DrawBoundingBox - draw black box around the page
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
#   box size is automatically computed
#
# Options:
# lw|line_width = <unit>    just which line width to use
#

package DrawBB;
use OM;
use sizes;
@ISA = qw(OM);
use strict;



sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("DrawBB");
	$self->{LINE_WIDTH} = 2; # default

	return bless($self,$class);
}

sub parse_options {
	my $self = shift;

	$self->log(4,"parsing options");
	while (my $str = shift) {
		if ($str =~ /^\s*(lw|line_width)\s*\=\s*(\w+)\s*$/) {
			my $w = unit_size($2);
			$self->die($w) if $w =~ /^ERROR/;
			$self->{LINE_WIDTH} = $w;
		} else {
			$self->die("bad option ($str)\n");
		}
	}
	$self->log(2,"line width is ",$self->{LINE_WIDTH});
}

sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};
	my $w = $self->{PAGE_WIDTH}{$page};
	my $h = $self->{PAGE_HEIGHT}{$page};

	$self->log(4,"writing page $page prolog (box drawing)");
	print $f 
		"userdict/PSTdict save put\nPSTmatrix setmatrix\n",
		"gsave\n",   # save graphics state
		$self->{LINE_WIDTH}," setlinewidth\n",
		"newpath ",
		"0 0 moveto ",  # start at lower left
		"0 $h lineto ", # bottom line
		"$w $h lineto ",  # right
		"$w 0 lineto ",   # top
		"0 0 lineto ",    # left
		"stroke\n",   # draw it
		"grestore\n", # restore graph state 
                "userdict/PSTmatrix matrix currentmatrix put\n",
                #"userdict/PSTclip{0 0 moveto\n",
                #$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
                #"closepath}put initclip\n",
                "PSTxform concat\n",
		"PSTdict restore\n";


}


1;  # no init
