#
# PST module Reorder - shuffle pages in document
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
# options:
# m|modulo = <int>
# p|pages = (pg1,pg2,...)
#     where pg must be from 0..modulo-1
#     if preceeded by '_', then the page is counted from end
#
# it reads modulo pages and writes them as defined in "pages"


package Reord;
use OM;
@ISA = qw(OM);
use strict;
use integer;



sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("Reord");
	$self->{MODULO} = 1;  # default
	$self->{PAGE_SPEC} = {};
	$self->{ORIG_PAGE} = {};

	return bless($self, $class);
}

sub parse_options {
	my $self = shift;
	my $mdl;
	my $pgspc = 0;
	
	$self->log(4,"parsing options");
	while (my $str = shift) {
		foreach my $e (split /;/, $str) {
			if ($e =~ /^\s*(m|modulo)\s*\=\s*(\d+)\s*$/ && $2>0) {
				$self->{MODULO} = $mdl = $2;
			} elsif ($e =~ /^\s*(p|pages)\s*\=\s*\(([^\)]+)\)\s*$/) {
				my $j = 0;
				foreach my $p (split /,/, $2) {
					if ($p =~ /^\s*(_?(\d+))\s*$/) {
						$self->{PAGE_SPEC}{$j} = $1;
						$pgspc = 1;
					} else {
						$self->die("bad pagespec ($p)\n");
					}
					++ $j;
				}
			} else {
				$self->die("bad options ($e)\n");
			}
		}
	}
	$self->die("I need some pagespec") if not $pgspc;
	$self->log(2,"modulo is ",$self->{MODULO});
}

sub page_size {
	my $self = shift;
	my $page = shift;
	my $mdl = $self->{MODULO};
	my $pss = (keys %{$self->{PAGE_SPEC}});
	my $pp;
	
	my $pspc = $self->{PAGE_SPEC}{($page-1) % $pss};
	if ($pspc =~ /^_(.*)$/) {  # from end
		$pspc = $1;
		$pp = $self->{PREV_PAGES} - ((($page-1)/$pss)*$mdl +$pspc);
	} else { # from begining
		$pp = (($page-1)/$pss)*$mdl +$pspc +1;
	}

	return 0 if $pp<1 or $pp>$self->{PREV_PAGES};
	$self->{PAGE_WIDTH}{$page} = $self->{PREV_PAGE_WIDTH}{$pp};
	$self->{PAGE_HEIGHT}{$page} = $self->{PREV_PAGE_HEIGHT}{$pp};
	$self->{ORIG_PAGE}{$page} = $pp;   # we must remember which page it originally is
	return 1;
}


sub write_page {
	my $self = shift;
	my $page = shift;

	$self->previous->write_page($self->{ORIG_PAGE}{$page});
}



1; # no init...
