#
# PST module Scale - scale the size of page
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
# options:
# h|height = <unit>
# w|width = <unit>
# p|paper = <paper>
#    if the name of the unit is omitted, the value is taken as relative

package Scale;
use OM;
use sizes;
@ISA = qw(OM);
use strict;

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("Scale");
	$self->{WIDTH} = undef;
	$self->{HEIGHT} = undef;
	$self->{W_RELATIVE} = 0;
	$self->{H_RELATIVE} = 0;

	return bless($self,$class);
}

sub parse_options {
	my $self = shift;

	$self->log(4,"parsing options");
	while (my $str = shift) {
		foreach  (split /;/, $str) {
			if (/^\s*(w|width)\s*\=\s*((\d+(\.\d+)?)\s*(\w*))\s*$/) {
				my $u = unit_size($2);
				$u = $2 and $self->{W_RELATIVE} = 1 if $5 eq "";
				$self->die($u) if $u =~ /^ERROR/;
				$self->{WIDTH} = $u;
			} elsif (/^\s*(h|height)\s*\=\s*((\d+(\.\d+)?)\s*(\w*))\s*$/) {
				my $u = unit_size($2);
				$u = $2 and $self->{H_RELATIVE} = 1 if $5 eq "";
				$self->die($u) if $u =~ /^ERROR/;
				$self->{HEIGHT} = $u;
			} elsif (/^\s*(p|paper)\s*\=\s*(\w+)\s*$/) {
				my @r = paper_size($2);
				$self->die($r[0]) if $r[0] =~ /^ERROR/;
				($self->{WIDTH},$self->{HEIGHT}) = @r;
				$self->{W_RELATIVE} = $self->{H_RELATIVE} = 0;
			} else {
				$self->die("bad option ($_)\n");
			}
		}
	}
	$self->log(2,"scaling to ", $self->{W_RELATIVE} ? "*" : "", $self->{WIDTH},
		"x", $self->{H_RELATIVE} ? "*" : "", $self->{HEIGHT});
}

sub page_size {
	my $self = shift;
	my $page = shift;

	return 0 if $page<1 or $page>$self->{PREV_PAGES};
	if ($self->{W_RELATIVE}) {
		$self->{PAGE_WIDTH}{$page} = $self->{PREV_PAGE_WIDTH}{$page} * $self->{WIDTH};
	} else { # absolute
		$self->{PAGE_WIDTH}{$page} = $self->{WIDTH};
	}
	if ($self->{H_RELATIVE}) {
		$self->{PAGE_HEIGHT}{$page} = $self->{PREV_PAGE_HEIGHT}{$page} * $self->{HEIGHT};
	} else { # absolute
		$self->{PAGE_HEIGHT}{$page} = $self->{HEIGHT};
	}
	return 1;
}

sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $h = $self->{OUTPUT};
	my $x = $self->{WIDTH};
	my $y = $self->{HEIGHT};
	my $pw = $self->{PAGE_WIDTH}{$page};
	my $ph = $self->{PAGE_HEIGHT}{$page};


	if (not $self->{W_RELATIVE}) {
		$x /= $self->{PREV_PAGE_WIDTH}{$page};
	}
	if (not $self->{H_RELATIVE}) {
		$y /= $self->{PREV_PAGE_HEIGHT}{$page};
	}

	print $h "userdict/PSTdict save put\nPSTmatrix setmatrix\n",
		"$x $y scale\n",
		"userdict/PSTmatrix matrix currentmatrix put\n",
		#"userdict/PSTclip{0 0 moveto\n",
                #$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
                #"closepath}put initclip\n",
                "PSTxform concat\n";
}

sub write_page_epilog {
	my $self = shift;
	my $f = $self->{OUTPUT};

	print $f "PSTdict restore\n";
}



1; # no init...
